=== modified file 'add_window.c'
--- add_window.c	2020-12-31 13:07:38 +0000
+++ add_window.c	2021-03-06 15:11:28 +0000
@@ -42,7 +42,6 @@
 #ifdef EWMH
 # include "ewmh_atoms.h"
 #endif
-#include "functions_internal.h"
 #include "gram.tab.h"
 #include "icons.h"
 #include "iconmgr.h"
@@ -63,6 +62,7 @@
 #include "win_ops.h"
 #include "win_regions.h"
 #include "win_resize.h"
+#include "win_ring.h"
 #include "win_utils.h"
 #include "workspace_manager.h"
 #include "xparsegeometry.h"
@@ -444,7 +444,7 @@
 		AddWindowToRing(tmp_win);
 	}
 	else {
-		tmp_win->ring.next = tmp_win->ring.prev = NULL;
+		InitWindowNotOnRing(tmp_win);
 	}
 
 

=== modified file 'cmake_files/basic_vars.cmake'
--- cmake_files/basic_vars.cmake	2018-11-12 03:59:16 +0000
+++ cmake_files/basic_vars.cmake	2021-03-06 15:11:28 +0000
@@ -70,6 +70,7 @@
 	win_ops.c
 	win_regions.c
 	win_resize.c
+	win_ring.c
 	win_utils.c
 	windowbox.c
 	workspace_config.c

=== modified file 'event_handlers.c'
--- event_handlers.c	2020-12-31 13:07:38 +0000
+++ event_handlers.c	2021-03-06 15:11:28 +0000
@@ -43,7 +43,6 @@
 #include "event_internal.h"
 #include "event_names.h"
 #include "functions.h"
-#include "functions_internal.h"
 #include "functions_defs.h"
 #include "gram.tab.h"
 #include "iconmgr.h"
@@ -61,6 +60,7 @@
 #include "win_ops.h"
 #include "win_regions.h"
 #include "win_resize.h"
+#include "win_ring.h"
 #include "win_utils.h"
 #include "workspace_manager.h"
 #include "workspace_utils.h"
@@ -3358,7 +3358,7 @@
 			/*
 			 * set ring leader
 			 */
-			if(Tmp_win->ring.next && (!enter_flag || raise_win == enter_win)) {
+			if(WindowIsOnRing(Tmp_win) && (!enter_flag || raise_win == enter_win)) {
 				Scr->RingLeader = Tmp_win;
 			}
 			XSync(dpy, 0);

=== modified file 'functions_internal.h'
--- functions_internal.h	2020-12-31 13:07:38 +0000
+++ functions_internal.h	2021-03-06 15:11:28 +0000
@@ -191,8 +191,4 @@
 /* Several places need to frob this to leave the cursor alone */
 extern bool func_reset_cursor;
 
-/* Exported from functions_warp.c to manage the window ring */
-void UnlinkWindowFromRing(TwmWindow *win);
-void AddWindowToRing(TwmWindow *win);
-
 #endif /* _CTWM_FUNCTIONS_INTERNAL_H */

=== modified file 'functions_warp.c'
--- functions_warp.c	2020-12-31 13:38:50 +0000
+++ functions_warp.c	2021-03-06 15:11:28 +0000
@@ -20,10 +20,10 @@
 #include "otp.h"
 #include "screen.h"
 #include "win_iconify.h"
+#include "win_ring.h"
 #include "win_utils.h"
 
 
-bool WindowIsOnRing(TwmWindow *win);
 static void WarpAlongRing(XButtonEvent *ev, bool forward);
 
 
@@ -121,67 +121,10 @@
 	}
 }
 
-void
-UnlinkWindowFromRing(TwmWindow *win)
-{
-	TwmWindow *prev = win->ring.prev;
-	TwmWindow *next = win->ring.next;
-
-	/*
-	* 1. Unlink window
-	* 2. If window was only thing in ring, null out ring
-	* 3. If window was ring leader, set to next (or null)
-	*
-	* If the window is the only one in the ring, prev == next == win,
-	* so the unlinking effectively is a NOP, but that doesn't matter.
-	*/
-	prev->ring.next = next;
-	next->ring.prev = prev;
-
-	win->ring.next = win->ring.prev = NULL;
-
-	if(Scr->Ring == win) {
-		Scr->Ring = (next != win ? next : NULL);
-	}
-
-	if(!Scr->Ring || Scr->RingLeader == win) {
-		Scr->RingLeader = Scr->Ring;
-	}
-}
-
-static void
-AddWindowToRingUnchecked(TwmWindow *win, TwmWindow *after)
-{
-	TwmWindow *before = after->ring.next;
-
-	win->ring.next = before;
-	win->ring.prev = after;
-
-	after->ring.next = win;
-	before->ring.prev = win;
-}
-
-void
-AddWindowToRing(TwmWindow *win)
-{
-	if(Scr->Ring) {
-		AddWindowToRingUnchecked(win, Scr->Ring);
-	}
-	else {
-		win->ring.next = win->ring.prev = Scr->Ring = win;
-	}
-}
-
-bool
-WindowIsOnRing(TwmWindow *win)
-{
-	return win && win->ring.next;
-}
-
 /* Taken from vtwm version 5.3 */
 DFHANDLER(ring)
 {
-	if(tmp_win->ring.next || tmp_win->ring.prev) {
+	if(WindowIsOnRing(tmp_win)) {
 		/* It's in the ring, let's take it out. */
 		UnlinkWindowFromRing(tmp_win);
 	}

=== added file 'win_ring.c'
--- win_ring.c	1970-01-01 00:00:00 +0000
+++ win_ring.c	2021-03-06 15:11:28 +0000
@@ -0,0 +1,59 @@
+/*
+ * Functions related to the window ring.
+ */
+
+#include "ctwm.h"
+
+#include "screen.h"
+#include "win_ring.h"
+
+void
+UnlinkWindowFromRing(TwmWindow *win)
+{
+	TwmWindow *prev = win->ring.prev;
+	TwmWindow *next = win->ring.next;
+
+	/*
+	* 1. Unlink window
+	* 2. If window was only thing in ring, null out ring
+	* 3. If window was ring leader, set to next (or null)
+	*
+	* If the window is the only one in the ring, prev == next == win,
+	* so the unlinking effectively is a NOP, but that doesn't matter.
+	*/
+	prev->ring.next = next;
+	next->ring.prev = prev;
+
+	win->ring.next = win->ring.prev = NULL;
+
+	if(Scr->Ring == win) {
+		Scr->Ring = (next != win ? next : NULL);
+	}
+
+	if(!Scr->Ring || Scr->RingLeader == win) {
+		Scr->RingLeader = Scr->Ring;
+	}
+}
+
+static void
+AddWindowToRingUnchecked(TwmWindow *win, TwmWindow *after)
+{
+	TwmWindow *before = after->ring.next;
+
+	win->ring.next = before;
+	win->ring.prev = after;
+
+	after->ring.next = win;
+	before->ring.prev = win;
+}
+
+void
+AddWindowToRing(TwmWindow *win)
+{
+	if(Scr->Ring) {
+		AddWindowToRingUnchecked(win, Scr->Ring);
+	}
+	else {
+		win->ring.next = win->ring.prev = Scr->Ring = win;
+	}
+}

=== added file 'win_ring.h'
--- win_ring.h	1970-01-01 00:00:00 +0000
+++ win_ring.h	2021-03-06 15:11:28 +0000
@@ -0,0 +1,13 @@
+/*
+ * Functions related to the window ring.
+ */
+#ifndef _CTWM_WIN_RING_H
+#define _CTWM_WIN_RING_H
+
+void UnlinkWindowFromRing(TwmWindow *win);
+void AddWindowToRing(TwmWindow *win);
+
+#define WindowIsOnRing(win) ((win) && (win)->ring.next)
+#define InitWindowNotOnRing(win) ((win)->ring.next = (win)->ring.prev = NULL)
+
+#endif

